/*
 * @(#)TargetManager.java	1.1 97/09/09
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */
package com.sun.java.swing.target;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;

/**
 * A service for dynamically finding targets. Mainly used by
 * TargetedActions, but not strictly tied to them.
 *
 * @version 
 * @author Ray Ryan
 */
public interface TargetManager {
    public void setPreTargets(int index, Object target);
    public Object getPreTargets(int index);
    public void setPreTargets(Object[] targets);
    public Object[] getPreTargets();
    
    public void setPostTargets(int index, Object target);
    public Object getPostTargets(int index);
    public void setPostTargets(Object[] targets);
    public Object[] getPostTargets();

    public void removeChangeListener(ChangeListener l);
    public void addChangeListener(ChangeListener l);

    public void registerWindow(Window w);

    /**
     * Present candidate targets to the given Iterator
     */
    public void findTarget(Iterator c);

    public class Util {
	// PENDING(rjrjr) Do the thread group stuff
	protected static TargetManager theTm = null;

	// PENDING(rjrjr) Why do the dummy thing? Why not default to
	// an instance of TargetManager.Implementation?
	/**
	 * Return the TargetManager for the current ThreadGroup. If
	 * none has been registered with setTargetManager, return an
	 * instance of a dummy implementation that does nothing.
	 */
 	public static TargetManager getTargetManager() {
	    if (theTm == null) {
		theTm = new Dummy();
	    }
	    return theTm;
	}

	/**
	 * Set the TargetManager for the current ThreadGroup
	 */
	public static void setTargetManager(TargetManager tm) {
	    theTm = tm;
	}

	/**
	 * A utility function for resolving target delegates, for use 
	 * by implementations of TargetManager in their
	 * findTargetAndContinue methods.
	 * 
	 * <p>If target has a delegate (indicated by implementing the
	 * Target interface and giving a non-null return to
	 * getTargetDelegate) present that to the iterator, then present
	 * the target itself. Returns true if all candidates are
	 * rejected and the search must continue, false if the search
	 * is done.
	 * 
	 * <p>This is a recursive test. The delegate may delegate in
	 * turn, up to a limit of 100 delegates. A delegate always
	 * gets first crack.
	 */
	static public boolean descendTargetForIteratorAndContinue(
			          Object target, Iterator c) {
	    return descendTargetForIteratorWatchDepth(target, c, 0);
	}

	/**
	 * The actual recursing part of descendTargetForIteratorAndContinue
	 */
        static protected boolean descendTargetForIteratorWatchDepth(
				     Object o, Iterator c, int depth) {
	    if (depth++ > 100) {
		System.out.println("Too many target delegates");
		return false;
	    }

	    System.out.print("d"+depth);

	    if (o == null) {
		return true;
	    } else if (!(o instanceof Target)) {
		return c.setNextAndContinue(o);
	    } else {
		return descendTargetForIteratorWatchDepth(
		           ((Target)o).getTargetDelegate(), c, depth) 
		    && c.setNextAndContinue(o);
	    }
	}
    }
    

    class Dummy implements TargetManager {
	Dummy() {}
	    
	public void setPreTargets(int index, Object target) {}
	public Object getPreTargets(int index) { return null; }
	public void setPreTargets(Object[] targets) {}
	public Object[] getPreTargets() { return null; }
    
	public void setPostTargets(int index, Object target) {}
	public Object getPostTargets(int index) { return null; }
	public void setPostTargets(Object[] targets) {}
	public Object[] getPostTargets() { return null; }

	public void addChangeListener(ChangeListener l) {}
	public void removeChangeListener(ChangeListener l) {}

	public void registerWindow(Window w) {}

	public void findTarget(Iterator c) {}
    }

    abstract class Abstract implements TargetManager {
	protected Vector preList = null;
	protected Vector postList = null;

	protected Vector getPreList() {
	    if (preList == null) {
		preList = new Vector();
	    }

	    return preList;
	}

	public void setPreTargets(int index, Object target) {
	    this.getPreList().setElementAt(target, index);
	}
	
	public Object getPreTargets(int index) {
	    return this.getPreList().elementAt(index);
	}
	
	public void setPreTargets(Object[] targets) {
	    int count = targets.length;

	    if (count == 0) {
		preList = null;
	    } else {
		preList = new Vector(targets.length);
		for (int i = 0; i < count; i++) {
		    preList.setElementAt(targets[i], i);
		}
	    }
	}
	
	public Object[] getPreTargets() {
	    int count = 0;
	    if (preList != null) {
		Object[] targets = new Object[preList.size()];
		preList.copyInto(targets);
		return targets;
	    } else {
		return null;
	    }
	}
    
	protected Vector getPostList() {
	    if (postList == null) {
		postList = new Vector();
	    }

	    return postList;
	}

	public void setPostTargets(int index, Object target) {
	    this.getPostList().setElementAt(target, index);
	}
	
	public Object getPostTargets(int index) {
	    return this.getPostList().elementAt(index);
	}
	
	public void setPostTargets(Object[] targets) {
	    int count = targets.length;

	    if (count == 0) {
		postList = null;
	    } else {
		postList = new Vector(targets.length);
		for (int i = 0; i < count; i++) {
		    postList.setElementAt(targets[i], i);
		}
	    }
	}
	
	public Object[] getPostTargets() {
	    int count = 0;
	    if (postList != null) {
		Object[] targets = new Object[postList.size()];
		postList.copyInto(targets);
		return targets;
	    } else {
		return null;
	    }
	}
	public void findTarget(Iterator c) {
	    System.out.print("Finding a target with " + c + "...");
	    boolean keepGoing = true;
	    if (preList != null) {
		int count = preList.size();
		for (int i=0; keepGoing && i < count; i++) {
		    System.out.print("pre["+i+"]...");
		    keepGoing = Util.descendTargetForIteratorAndContinue(
				        preList.elementAt(i++), c);
		}
	    }
		
	    if (keepGoing && this.findTargetAndContinue(c) &&
		postList != null) {
		int count = postList.size();
		for (int i=0; keepGoing && i < count; i++) {
		    System.out.print("post["+i+"]...");
		    keepGoing = Util.descendTargetForIteratorAndContinue(
					postList.elementAt(i++), c);
		}
	    }
	}

	// PENDING(rjrjr) Implement this!
	public void addChangeListener(ChangeListener l) {}
	public void removeChangeListener(ChangeListener l) {}

	// PENDING(rjrjr) What a *terrible* name
	protected abstract boolean findTargetAndContinue(Iterator c);
	
    }

    public class Implementation extends Abstract
    {
	protected WindowAdapter windowAdapter = null;
	protected Vector windowsVector = null;

	public Implementation() { super(); }
	    
	protected WindowAdapter getWindowAdapter() {
	    if (windowAdapter == null) windowAdapter = new MyWindowAdapter();
	    return windowAdapter;
	}

	protected Vector getWindowsVector() {
	    if (windowsVector == null) windowsVector = new Vector();
	    return windowsVector;
	}

	public void registerWindow(Window w) {
	    getWindowsVector().insertElementAt(w, 0);
	    w.addWindowListener(getWindowAdapter());
	}

	protected boolean findTargetAndContinue(Iterator c) {
	    Vector windows = getWindowsVector();

	    if (windows.size() > 0) {
		Component fc = ((Window)windows.elementAt(0)).getFocusOwner();
		while (fc != null) {
		    if (!Util.descendTargetForIteratorAndContinue(fc, c)) {
			return false;
		    }
		    fc = fc.getParent();
		}
	    }

	    return true;
	}

	class MyWindowAdapter extends WindowAdapter {
	    /**
	     * Invoked when a window has been opened.
	     */
	    public void windowOpened(WindowEvent e) {
		Vector windows = getWindowsVector();
		Window w = e.getWindow();
		if (windows.contains(w)) windows.removeElement(w);
		windows.insertElementAt(w, 0);
	    }

	    /**
	     * Invoked when a window is in the process of being closed.
	     * The close operation can be overridden at this point.
	     */
	    public void windowClosing(WindowEvent e) {}

	    /**
	     * Invoked when a window has been closed.
	     */
	    public void windowClosed(WindowEvent e) {
		Vector windows = getWindowsVector();
		Window w = e.getWindow();
		if (windows.contains(w)) windows.removeElement(w);
	    }

	    /**
	     * Invoked when a window is iconified.
	     */
	    public void windowIconified(WindowEvent e) {
		Vector windows = getWindowsVector();
		Window w = e.getWindow();
		if (windows.contains(w)) windows.removeElement(w);
	    }

	    /**
	     * Invoked when a window is de-iconified.
	     */
	    public void windowDeiconified(WindowEvent e) {
		Vector windows = getWindowsVector();
		Window w = e.getWindow();
		if (windows.contains(w)) windows.removeElement(w);
		windows.insertElementAt(w, 0);
	    }

	    /**
	     * Invoked when a window is activated.
	     */
	    public void windowActivated(WindowEvent e) {
		Vector windows = getWindowsVector();
		Window w = e.getWindow();
		if (windows.contains(w)) windows.removeElement(w);
		windows.insertElementAt(w, 0);
	    }

	    /**
	     * Invoked when a window is de-activated.
	     */
	    public void windowDeactivated(WindowEvent e) {}
	
	}
    }
}
