/*
 * @(#)Trigger.java	1.1 97/09/09
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */
package com.sun.java.swing.target;

import java.awt.event.*;
import com.sun.java.swing.event.*;

/**
 * An object that can carry an event to a target. A default
 * implementation is provided that can dynamically find its target
 * with a TargetManager.
 *
 * @version 
 * @author Ray Ryan
 */
public interface Trigger extends Cloneable {
    boolean canFire(Object triggerOwner);
    void fire(java.util.EventObject event);

    Object clone();
    Object getTarget();
    Object getResolvedTarget();
    boolean canSetTarget(Object candidate);
    void setTarget(Object target);

    public abstract class Abstract implements Trigger {
	protected Object target;
    
	public Abstract() {
	    this(null);
	}
    
	public Abstract(Object target) {
	    super();
	    this.setTarget(target);
	}

	public Object getTarget() {
	    return target;
	}

	public Object clone() {
	    Object clone = null;
	    try {
		clone = super.clone();
	    } catch (java.lang.CloneNotSupportedException e) {}
	    return clone;
	}

	/**
	 * If rawTarget is null, find a target on the default TargetManager
	 * for this threadGroup<br>
	 * If rawTarget is a TargetManager, find a target on it<br>
	 * If rawTarget is any other kind of object, return it.
	 *
	 * <p>TargetManagers are searched by walking through their
	 * targets, handing each to this.canSetTarget(), and stopping if
	 * true is returned. If none pass that test, return null.
	 */
	public Object getResolvedTarget() {
	    System.out.print("getting target: ");
	    Object tgt = this.getTarget();
	    if (tgt == null) {
		tgt = TargetManager.Util.getTargetManager();
	    }

	    if (tgt instanceof TargetManager) {
		TargetManager tm = (TargetManager)tgt;
		final Object[] result = {null};

		tm.findTarget(new Iterator() {
		    public boolean setNextAndContinue(Object next) {
			System.out.print(next.toString() + "? ");
			if (Trigger.this.canSetTarget(next)) {
			    System.out.print("Yes!  ");
			    result[0] = next;
			    return false;
			}
			System.out.print("No...");
			return true;
		    }
		});
		tgt = result[0];
	    }

	    System.out.println(tgt != null ? "found it" : "none.");
	    return tgt;
	}

	public void setTarget(Object target) {
	    this.target = target;
	}

	public boolean canSetTarget(Object candidate) {
	    return true;
	}
	
	/**
	 * Called during updateEnabled() to determine if this action
	 * should be enabled or not.
	 * 
	 * <p>Subclasses can override to determine if the current target
	 * is able to handle this action at this time. For example, the
	 * standard Paste command implements this as something like
	 * <code><pre> 
	 * {
	 *     Paster t = (Paster)this.getResolvedTarget();
	 *     return t != null && t.canPaste();
	 * }</pre></code>
	 *
	 * <p>This default method returns this.getResolvedTarget() != null
	 */
	public boolean canFire(Object triggerOwner) {
	    return this.getResolvedTarget() != null;
	}

	public abstract void fireEventAtTarget(java.util.EventObject e,
					       Object target);

	public void fire(java.util.EventObject e) {
	    Object tgt = this.getResolvedTarget();
	    if (tgt != null) this.fireEventAtTarget(e, tgt);
	    else System.out.println("fired at null target");
	}
    }
}
