/*
 * @(#)TriggerAction.java	1.1 97/09/09
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */
package com.sun.java.swing.target;

import java.beans.*;
import java.awt.event.*;
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;

/**
 * An Action that fires a Trigger when it receives an ActionEvent, and
 * which therefore can dispatch events to trigger targets via a
 * TargetManager. 
 *
 * A TriggerAction can monitor a TargetManager for ChangeEvents
 * (thrown when its target pool changes, or when it is sent
 * invalidateTargets()) and update its enabled state appropriately.
 * 
 * @version 
 * @author Ray Ryan
 */

public class TriggerAction extends AbstractAction
implements ChangeListener, Cloneable {
    protected Trigger trigger;
    protected TargetManager monitoredTargetMananger;
    
    public TriggerAction() {
	this(null, null, null);
    }
    
    public TriggerAction(Trigger trigger) {
	this(null, null, trigger);
    }

    public TriggerAction(String name, Icon icon, Trigger trigger) {
	super(name, icon);
	this.setTrigger(trigger);
    }

    public Object clone() {
	TriggerAction clone = null;
	try {
	    clone = (TriggerAction)super.clone();
	} catch (java.lang.CloneNotSupportedException e) {}
	
	clone.trigger = (Trigger)this.trigger.clone();
	return clone;
    }

    public Trigger getTrigger() {
	return trigger;
    }

    public void setTrigger(Trigger newTrigger) {
	this.trigger = newTrigger;
	Object target = newTrigger.getTarget();

	if (monitoredTargetMananger != target) {
	    monitoredTargetMananger.removeChangeListener(this);
	    monitoredTargetMananger = null;
	}
	
	if (target instanceof TargetManager) {
	    monitoredTargetMananger = (TargetManager)target;
	    monitoredTargetMananger.addChangeListener(this);
	}
	this.updateEnabled();
    }

    public void setTarget(Object target) {
	this.getTrigger().setTarget(target);
    }

    public Object getTarget() {
	return this.getTrigger().getTarget();
    }

    public Object getResolvedTarget() {
	return this.getTrigger().getResolvedTarget();
    }

    /**
     * Sets the enabled status of this TriggerAction based on
     * isTargetReceptive(), emitting appropriate changeEvents.
     *
     * <p> This method is called by stateChanged(ChangeEvent)
     */
    public void updateEnabled() {
	// changeSupport is the PropertyChangeSupport maintained by
	// AbstractAction, my superclass. If it is null, I have no
	// listeners, so why bother enabling or disabling?
	if (changeSupport != null) {
	    super.setEnabled(trigger.canFire(this));
	}
    }

    public void addPropertyChangeListener(PropertyChangeListener l) {
	boolean needUpdate = changeSupport == null;
	super.addPropertyChangeListener(l);
	if (needUpdate) this.updateEnabled();
    }

    public void actionPerformed(ActionEvent e) {
	this.getTrigger().fire(e);
    }

    public void stateChanged(ChangeEvent e) {
	this.updateEnabled();
    }
}
